# Generated by Django 4.2.27 on 2026-03-02 00:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("aacampaign", "0006_switch_to_eve_sde"),
        ("eveonline", "0015_factions"),
    ]

    operations = [
        migrations.AddField(
            model_name="campaignmember",
            name="faction",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="eveonline.evefactioninfo",
            ),
        ),
        migrations.AddField(
            model_name="campaigntarget",
            name="faction",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="eveonline.evefactioninfo",
            ),
        ),
    ]
