# Generated by Django 4.2.23 on 2026-01-24 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aacampaign', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='campaignkillmail',
            name='ship_type_id',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='campaignkillmail',
            name='ship_type_name',
            field=models.CharField(default='', max_length=255),
        ),
    ]
